@@ -8,6 +8,7 @@ use std::{
 };
 
 use crate::{
+    cache::CacheConfigSource,
     err::ConfigError,
     impl_cache,
     key::{CacheString, ConfigKey, PartialKeyIter},
@@ -219,8 +220,9 @@ impl<'a> ConfigContext<'a> {
         ConfigError::ConfigNotFound(self.current_key())
     }
 
+    /// Parse config value error.
     #[inline]
-    pub(crate) fn parse_error(&self, value: &str) -> ConfigError {
+    pub fn parse_error(&self, value: &str) -> ConfigError {
         ConfigError::ConfigParseError(self.current_key(), value.to_owned())
     }
 
@@ -313,33 +315,43 @@ impl Configuration {
         mut self,
         loader: L,
     ) -> Result<Self, ConfigError> {
+        let loader = CacheConfigSource::new(loader);
         loader.load(&mut self.source.prefixed())?;
         self.loaders.push(Box::new(loader));
         Ok(self)
     }
 
     #[inline]
-    fn reload(&self) -> Result<Configuration, ConfigError> {
+    fn reload(&self) -> Result<(bool, Configuration), ConfigError> {
         let mut s = Configuration::new();
-        let c = &mut s.source.prefixed();
-        for l in self.loaders.iter() {
-            l.load(c)?;
+        let mut refreshed = false;
+        for i in self.loaders.iter() {
+            if i.refreshable()? {
+                refreshed = true;
+            }
         }
-        self.source.refs.refresh(&s)?;
-        Ok(s)
+        if refreshed {
+            let c = &mut s.source.prefixed();
+            for i in self.loaders.iter() {
+                i.load(c)?;
+            }
+            self.source.refs.refresh(&s)?;
+        }
+        Ok((refreshed, s))
     }
 
     /// Refresh all [RefValue](struct.RefValue.html)s without change [`Configuration`] itself.
-    pub fn refresh_ref(&self) -> Result<(), ConfigError> {
-        let _ = self.reload()?;
-        Ok(())
+    pub fn refresh_ref(&self) -> Result<bool, ConfigError> {
+        Ok(self.reload()?.0)
     }
 
     /// Refresh all [RefValue](struct.RefValue.html)s and [`Configuration`] itself.
-    pub fn refresh(&mut self) -> Result<(), ConfigError> {
-        let c = self.reload()?;
-        self.source.value = c.source.value;
-        Ok(())
+    pub fn refresh(&mut self) -> Result<bool, ConfigError> {
+        let (x, c) = self.reload()?;
+        if x {
+            self.source.value = c.source.value;
+        }
+        Ok(x)
     }
 
     /// Get config from configuration by key, see [`ConfigKey`] for the key's pattern details.
