@@ -123,7 +123,7 @@ macro_rules! into_config_value_le {
 into_config_value_le!(Int = i64: u8, u16, u32, i8, i16, i32, i64);
 into_config_value_le!(Float = f64: f32, f64);
 
-macro_rules! into_config_value {
+macro_rules! into_config_value_u {
     ($($x:ident),*) => {$(
         impl<'a> Into<ConfigValue<'a>> for $x {
             fn into(self) -> ConfigValue<'a> {
@@ -136,7 +136,22 @@ macro_rules! into_config_value {
     };
 }
 
-into_config_value!(u64, u128, usize, i128, isize);
+into_config_value_u!(u64, u128, usize);
+
+macro_rules! into_config_value {
+    ($($x:ident),*) => {$(
+        impl<'a> Into<ConfigValue<'a>> for $x {
+            fn into(self) -> ConfigValue<'a> {
+                if self <= i64::MAX as $x && self>= i64::MIN as $x {
+                    return ConfigValue::Int(self as i64);
+                }
+                ConfigValue::Str(self.to_string())
+            }
+        })*
+    };
+}
+
+into_config_value!(i128, isize);
 
 impl<'a> Into<ConfigValue<'a>> for bool {
     fn into(self) -> ConfigValue<'a> {
@@ -256,6 +271,8 @@ impl FromValue for String {
     }
 }
 
+/// Get from string.
+#[doc(hidden)]
 pub trait FromStringValue: Sized + Any {
     fn from_str_value(context: &mut ConfigContext<'_>, value: &str) -> Result<Self, ConfigError>;
 }
@@ -437,8 +454,8 @@ impl FromValue for Duration {
 #[macro_export]
 macro_rules! impl_enum {
     ($x:path {$($($k:pat)|* => $v:expr)+ }) => {
-        impl $crate::value::FromStringValue for $x {
-            fn from_str_value(context: &mut $crate::configuration::ConfigContext<'_>, value: &str) -> Result<Self, $crate::err::ConfigError> {
+        impl $crate::FromStringValue for $x {
+            fn from_str_value(context: &mut $crate::ConfigContext<'_>, value: &str) -> Result<Self, $crate::ConfigError> {
                 match &value.to_lowercase()[..] {
                     $($($k)|* => Ok($v),)+
                     _ => Err(context.parse_error(value)),
@@ -475,11 +492,14 @@ mod test {
         }
 
         #[allow(single_use_lifetimes)]
-        fn read<'a, T: FromValue>(
+        fn read<'a, T: FromConfig>(
             &mut self,
             val: impl Into<ConfigValue<'a>>,
         ) -> Result<T, ConfigError> {
-            T::from_value(&mut self.0.source.new_context(&mut self.1), val.into())
+            T::from_config(
+                &mut self.0.source.new_context(&mut self.1),
+                Some(val.into()),
+            )
         }
     }
 
@@ -561,6 +581,67 @@ mod test {
         should_eq!(context: i as i64 => i);
     }
 
+    macro_rules! num_into_test {
+        ($($fun:ident. $t:ty,)+) => {
+            $(
+            #[quickcheck]
+            fn $fun(i: $t) {
+                let v: ConfigValue<'static> = i.into();
+                match v {
+                    ConfigValue::Int(_) => {}
+                    _ => assert_eq!(true, false),
+                }
+            }
+            )+
+        };
+    }
+
+    num_into_test!(
+        u8_test.u8,
+        u16_test.u16,
+        u32_test.u32,
+        i8_test.i8,
+        i16_test.i16,
+        i32_test.i32,
+        i64_test.i64,
+    );
+
+    macro_rules! num_into_test_u {
+        ($($fun:ident. $t:ty),+) => {
+            $(
+            #[quickcheck]
+            fn $fun(i: $t) {
+                let v: ConfigValue<'static> = i.into();
+                match v {
+                    ConfigValue::Int(_) => assert_eq!(true,  i <= i64::MAX as $t),
+                    ConfigValue::Str(_) => assert_eq!(true, i > i64::MAX as $t),
+                    _ => assert_eq!(true, false),
+                }
+            }
+            )+
+        };
+    }
+
+    num_into_test_u!(u64_test.u64, u128_test.u128, usize_test.usize);
+
+    macro_rules! num_into_test_i {
+        ($($fun:ident. $t:ty),+) => {
+            $(
+            #[quickcheck]
+            fn $fun(i: $t) {
+                let v: ConfigValue<'static> = i.into();
+                match v {
+                    ConfigValue::Int(_) => assert_eq!(true,  i <= i64::MAX as $t && i>= i64::MIN as $t),
+                    ConfigValue::Str(_) => assert_eq!(true, i > i64::MAX as $t || i< i64::MIN as $t),
+                    _ => assert_eq!(true, false),
+                }
+            }
+            )+
+        };
+    }
+
+    num_into_test_i!(i128_test.i128, isize_test.isize);
+
     #[quickcheck]
     fn i64_tests(i: i64) {
         let mut context = TestContext::new();
@@ -618,4 +699,28 @@ mod test {
         buf.push("/var");
         should_eq!(context: "/var" as PathBuf => buf);
     }
+
+    #[test]
+    #[allow(unused_qualifications)]
+    fn option_test() {
+        let mut context = TestContext::new();
+        let x: Result<Option<Ordering>, ConfigError> = context.read("val");
+        assert_eq!(true, x.is_err());
+        match x.unwrap_err() {
+            ConfigError::ConfigParseError(_, _) => {}
+            _ => assert_eq!(true, false),
+        }
+    }
+
+    #[test]
+    #[allow(unused_qualifications)]
+    fn into_string_test() {
+        let mut context = TestContext::new();
+        let v: String = context.read(1.1f64).unwrap();
+        assert_eq!("1.1", v);
+        let v: String = context.read(1u8).unwrap();
+        assert_eq!("1", v);
+        let v: String = context.read(true).unwrap();
+        assert_eq!("true", v);
+    }
 }
